/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: XMLConfigurationInjector.java 3054 2008-04-30 15:41:13Z sauthieg $
 * --------------------------------------------------------------------------
 */
package org.ow2.easybeans.osgi.configuration;

import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.ow2.easybeans.component.api.EZBComponent;
import org.ow2.easybeans.server.Embedded;
import org.ow2.util.xmlconfig.XMLConfiguration;
import org.ow2.util.xmlconfig.XMLConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 * TODO: Class Description.
 *
 * @author David Alves
 * @version $Revision$
 */
public class XMLConfigurationInjector extends XMLConfiguration {

    /** The xml configuration. */
    private String xmlConfiguration;

    private EZBComponent componentObject;

    private Embedded embedded;

    /**
     * Instantiates a new XML configuration injector.
     *
     * @param componentClass the component class
     * @param xmlConfiguration the xml configuration
     *
     * @throws XMLConfigurationException the XML configuration exception
     */
    public XMLConfigurationInjector(final Class<? extends EZBComponent> componentClass, final String xmlConfiguration) throws XMLConfigurationException {
        super(null, "http://easybeans.ow2.org/xml/ns/mapping", "easybeans-mapping.xml");
        this.xmlConfiguration = xmlConfiguration;
        try {
            componentObject = componentClass.newInstance();
        } catch (Exception e) {
            throw new XMLConfigurationException("Could not instantiate component class: " + componentClass.getName(), e);
        }
        // TODO refactor so this doesn't need to configure the whole container
        embedded = new Embedded();
        configure(embedded);

    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.easybeans.xmlconfig.XMLConfiguration#configure(java.lang.Object)
     */
    @Override
    public void configure(final Object object) throws XMLConfigurationException {
        // No validation of the XML is made since it has been already performed
        Document componentConfigurationXML;
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            documentBuilderFactory.setNamespaceAware(true);
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            componentConfigurationXML = documentBuilder.parse(new InputSource(new StringReader(this.xmlConfiguration)));
        } catch (Exception e) {
            throw new XMLConfigurationException("Could not configure object from XML Configuration.", e);
        }
        Element rootElement = componentConfigurationXML.getDocumentElement();
        super.configure(object, rootElement, null);
    }

    public EZBComponent getConfiguredComponent(){
        for (EZBComponent component : embedded.getComponents().getEZBComponents()) {
            if (component.getClass().equals(componentObject.getClass())){
                return component;
            }
        }
        return null;
    }

}
